/*
 MotorController via Web Server

 Creates a server in the vehicle and connects to the specified WifiNetwork.
 Use the UART COM communication to check the status of connection and IP.
 By connecting thought a Web explorer inside the Network the movement of the vehicle can be controlled
 and the available networks are printed in the explorer showing the number of networks, the SSID and RSSI for each one.

 This example is written for a network using WPA encryption. For
 WEP or WPA, change the Wifi.begin() call accordingly.

 Circuit:
 * CC3200 WiFi LaunchPad a MSP430 LaunchPad

 created 15 April 2018
 by Javier Artacho Garcia
 */

#ifndef __CC3200R1M1RGC__
#include <SPI.h>
#endif
#include <WiFi.h>
#include <WiFiClient.h>
#include <Wire.h>
#include <Stdio.h>
#include "SCMD.h"
#include "HCSR04.h"
#include "configWIFI.h"
#include "configHardware.h"
#include "configSys.h"

volatile bool lContact = false;    //variable that stores if a left contact exists
volatile bool rContact = false;     //variable that stores if a right contact exists

long distToObst;    //variable that stores the distance measured

int keyIndex = 0;

SCMD mySCMD;
HCSR04 myDistSensor;

WiFiServer server(myPort);

// structure definitions used by the main code

struct ListNetArray{     //matrix of network Id and SSID and RSSI
    int Id;
    char SSID[32];
};

struct ListRSSIArray{    //matrix of network Id and RSSI for actual position
    int Id;
    int32_t RSSI;
};

ListNetArray ListOfNetworks[numNetwork] {};        //array of network Id-SSID
ListRSSIArray actualRSSInetworks[numNetwork] {};  //array of network Id-RSID for the present position

int idSession;      // variable that stores the session Id
int idPosition;      // variable that stores the position Id
int posAvai=0;        // variable that stores if the position is available(=0) or not(=1)

WiFiClient client;   //client in wifi network
IPAddress ip(AWSR_IPaddress); //fixed IP for the robot

// configuration for WIFI connection
extern char ssid[] = mySSID;          // your network name also called SSID
extern char password[] = myPassword;   // your network password

// configuration of the server where the data is sent
extern uint16_t port = myPort;     // port number of the server
extern IPAddress serverObj(myIPaddress);   // IP Address of the server



void setup() {

  // initialization of the pins
  Serial.begin(serialBaudRate);     // initialize serial communication
  pinMode(onLED, OUTPUT);      // set the LED pin mode - light is ON when the robot is scanning
  pinMode(leftContLED, INPUT);       // set the left contact pin mode
  pinMode(rightContLED, INPUT);       // set the right contact pin mode
  pinMode(trigPin, OUTPUT); //define trigger pin for distance sensor as output
  pinMode(echoPin, INPUT);  //define echo pin for distance sensor as input

  // initialization of the interruptions attached to the pins
  attachInterrupt(leftContactPIN,leftClash,FALLING);    //sets the interrupt for the left contact sensor on pin P2.6
  attachInterrupt(rightContactPIN,rightClash,FALLING);    //sets the interrupt for the right contact sensor on pin P2.7


  Wire.setModule(0);
  Wire.begin(addressI2C);     // join i2c bus with address #0x59
  mySCMD.reset();       //resets the motor driver
  
  wifiConnect();        // connects to the wifi network specified in the header

  //Enables the output driver hardware for controlling the motors
   Wire.beginTransmission(addressI2C);
   Wire.write(0x70);
   Wire.write(0x01);
   Wire.endTransmission();
}


void loop() {
  int i = 0;
  WiFiClient client = server.available();   // listen for incoming clients
  distToObst=myDistSensor.distObstacle();    //distance to obstacle

  // local server definition to connect externally and control the vehicle

  if (client) {                             // if you get a client,
    Serial.println("new client");           // print a message out the serial port
    char buffer[150] = {0};                 // make a buffer to hold incoming data
    while (client.connected()) {            // loop while the client's connected
      if (client.available()) {             // if there's bytes to read from the client,
        char c = client.read();             // read a byte, then
        Serial.write(c);                    // print it out the serial monitor
        if (c == '\n') {                    // if the byte is a newline character

          // if the current line is blank, you got two newline characters in a row.
          // that's the end of the client HTTP request, so send a response:
          if (strlen(buffer) == 0) {
            // HTTP headers always start with a response code (e.g. HTTP/1.1 200 OK)
            // and a content-type so the client knows what's coming, then a blank line:
            client.println("HTTP/1.1 200 OK");
            client.println("Content-type:text/html");
            client.println();

            // the content of the HTTP response follows the header:
            client.println("<html><head><title>Motor Controller</title></head><body align=center>");
            client.println("<h1 align=center><font color=\"red\">Welcome to the AUTONOMOUS WIFI SCANNER ROBOT Server</font></h1>");
            client.print("START SCANNING <button onclick=\"location.href='/T'\">START</button>");
            client.println(" <button onclick=\"location.href='/S'\">STOP</button><br>");

            // scan for nearby networks:
            int NumNetworks=ScanNetworks();   //scans for the number of networks available
            if (NumNetworks==-1) {              // if no network exists
                client.println("Couldn't get a wifi connection");  //shows an error
                while (true);
           }

            // The HTTP response ends with another blank line:
            client.println();
            // break out of the while loop:
            break;
          }
          else {      // if you got a newline, then clear the buffer:
            memset(buffer, 0, 150);
            i = 0;
          }
        }
        else if (c != '\r') {    // if you got anything else but a carriage return character,
          buffer[i++] = c;      // add it to the end of the currentLine
        }

        // Check to see if the client request was GET /T /S:
        if (endsWith(buffer, "GET /T")) {       // Command to start the movement
          client.println("<br>** Scanning Networks **");
          digitalWrite(onLED, HIGH);               // LED 75 states that the robot is scanning
          idSession=readSessionId();            // retrieves the Session Id

          int totalPos=detNumberPositions();    // determine the number of positions in the room and returns the value
          int lastTurn=0;                       // variable equal to '0' if last turn was to left and '1' if was to right

          idPosition=0;                         // sets initial position
          ScanNetworks();                      // scans for available networks and store their data
          sendNetworks();                       // sends the data stored to the WebServer
          sendRSSI();                           // sends RSSI measured to the WebServer
          sendPosAvailable();                  //sends the actual position including that is available

              //code for controlling the movement
              distToObst=myDistSensor.distObstacle();    //distance to obstacle
              Serial.println("distance:");
              Serial.print(distToObst);
              Serial.println(" cm");
              while (idPosition<=totalPos) {
                  mySCMD.reset();       //resets the motor driver
                  if (lContact==true || rContact==true) {   //checks if a contact exists and avoids the clash
                      lContact=false;                       //initializes the variable of left contact to false
                      rContact=false;                       //initializes the variable of right contact to false
                      sendPosAvailable();                  //sends the actual position including that is not available
                  }

                  if (lContact==false || rContact==false) {   //checks if a contact does not exists and initializes the vaiable
                      posAvai=0;                           //initializes the variable to 0
                  }
                  while (myDistSensor.distObstacle() < 40 && lastTurn==0)  {
                          mySCMD.moveMotors (RIGHT);                   //TURN RIGHT the robot
                          delay(1000);                         // wait 2 seconds
                          mySCMD.moveMotors (STOP);                   //STOP both motors
                          delay(1000);
                          posAvai=1;                           //the next position is not available
                          idPosition++;                         //increase the variable of position
                          sendPosAvailable();                  //sends the actual position including that is available
                          if (myDistSensor.distObstacle() >= 30) {              // if no object exists in front
                              lastTurn=1;                       // especifies that the last turn was to right
                              mySCMD.moveMotors (RIGHT);                   //TURN RIGHT the robot
                              delay(1000);                         // wait 2 seconds
                              mySCMD.moveMotors (STOP);                   //STOP both motors
                          }
                  }
                  while (myDistSensor.distObstacle() < 40 && lastTurn==1) {
                          mySCMD.moveMotors (LEFT);                   //TURN RIGHT the robot
                          delay(1000);                         // wait 2 seconds
                          mySCMD.moveMotors (STOP);                   //STOP both motors
                          delay(1000);
                          posAvai=1;                           //the next position is not available
                          idPosition++;                         //increase the variable of position
                          sendPosAvailable();                  //sends the actual position including that is available
                          if (myDistSensor.distObstacle() >= 30) {              // if no object exists in front
                              lastTurn=0;                       // especifies that the last turn was to left
                              mySCMD.moveMotors (LEFT);                   //TURN RIGHT the robot
                              delay(1000);                         // wait 2 seconds
                              mySCMD.moveMotors (STOP);                   //STOP both motors
                          }
                  }
                  distToObst=myDistSensor.distObstacle();    //distance to obstacle
                  if (distToObst>20) {                  //checks if no obstacles exists
                      posAvai=0;                           //initializes the variable to 0
                  }
                  idPosition++;                         //increase the variable of position
                  ScanNetworks();                      // scans for available networks and store their data
                  sendNetworks();                       // sends the data stored to the WebServer
                  sendRSSI();                           // sends RSSI measured to the WebServer
                  sendPosAvailable();                  //sends the actual position including that is available

                  mySCMD.moveMotors (FORWARD);                //MOVE FORWARD both motors
                  delay(1000);                         // wait 1 second
                  mySCMD.moveMotors (STOP);                   //STOP both motors

                  Serial.println("distance:");              //prints the distance measured in the terminal
                  Serial.print(distToObst);
                  Serial.println(" cm");

                  if (distToObst >= 200 || distToObst <= 0){
                      Serial.println("Out of range");
                      mySCMD.moveMotors (STOP);                   //STOP both motors
                      for (int k=0; k<20;k++) {                   //LED blinking to indicate an error
                          digitalWrite(onLED, LOW);               // LED OFF
                          delay(500);                             // wait 0.5 seconds
                          digitalWrite(onLED, HIGH);               // LED ON
                      }
                  }
                  else {
                      Serial.print(distToObst);         //prints the distance measured in the terminal
                      Serial.println(" cm");
                  }
              }
        }

        if (endsWith(buffer, "GET /S")) {           // GET /S stop both motors
          digitalWrite(onLED, LOW);
          mySCMD.moveMotors (STOP);                   //STOP both motors
        }
      }
    }
    // close the connection:
    client.stop();
    Serial.println("client disconnected");
  }
}
/***************************************************************************************
****NEXT THE FUNCTIONS DEFINITION*******************************************************
***************************************************************************************/
//
//a way to check if one array ends with another array
//
boolean endsWith(char* inString, const char* compString) {
  int compLength = strlen(compString);
  int strLength = strlen(inString);
  
  //compare the last "compLength" values of the inString
  int i;
  for (i = 0; i < compLength; i++) {
    char a = inString[(strLength - 1) - i];
    char b = compString[(compLength - 1) - i];
    if (a != b) {
      return false;
    }
  }
  return true;
}

//function that prints through the serial the information related to the WIFI connection

void printWifiStatus() {
  Serial.print("SSID: ");
  Serial.println(WiFi.SSID());  // print the SSID of the network you're attached to

  // print your WiFi IP address:
  IPAddress ip = WiFi.localIP();
  Serial.print("IP Address: ");
  Serial.println(ip);

  // print the received signal strength:
  long rssi = WiFi.RSSI();
  Serial.print("signal strength (RSSI):");
  Serial.print(rssi);
  Serial.println(" dBm");
  // print where to go in a browser:
  Serial.print("To see this page in action, open a browser to http://");
  Serial.println(ip);
}

//function that scan for available networks and returns the number of networks.
//this function stores also the Network Id (for internal use only), name(SSID) and RSSI

int ScanNetworks(void){

      int numSsid = WiFi.scanNetworks();        //scans to know the number of networks (stored in numSsid)
      if (numSsid == -1)                        //if no network detected
      {
        return numSsid;                         //returns the error
        while (true);
      }
      int netPos=0;                             //variable to search inside the struct ListOfNetworks
      int RSSIpos=0;                            //variable to search inside the struct actualRSSInetworks
      if (ListOfNetworks[0].Id==NULL) {                       // if first initialization, read networks and store
          for (int thisNet = 0; thisNet < numSsid; thisNet++) {
                  ListOfNetworks[thisNet].Id=thisNet+1;
                  strcpy(ListOfNetworks[thisNet].SSID, WiFi.SSID(thisNet));     //adds new network to the array: SSID
                  actualRSSInetworks[thisNet].Id=thisNet+1;                     //Id of the network
                  actualRSSInetworks[thisNet].RSSI=WiFi.RSSI(thisNet);          //RSSI measured for the network

                }
       }
      while (ListOfNetworks[netPos].Id!=NULL)  {                            // checks if we have reached the end of the database

          for (int thisNet = 0; thisNet < numSsid; thisNet++) {             // loop into the networks detected
              if (ListOfNetworks[netPos].SSID==WiFi.SSID(thisNet)) {        // if the current network's SSID is the same as current network
                  actualRSSInetworks[RSSIpos].Id=ListOfNetworks[netPos].Id; // stores the network ID into actualRSSInetworks[]
                  actualRSSInetworks[RSSIpos].RSSI=WiFi.RSSI(thisNet);      // stores the RSSI for this network into actualRSSInetworks[]
                  RSSIpos++;

              } else if (ListOfNetworks[netPos+1].Id==NULL && ListOfNetworks[netPos].SSID!=WiFi.SSID(thisNet)){ //in case the network is not found, it is added at the end of the array
                   strcpy(ListOfNetworks[netPos+1].SSID, WiFi.SSID(thisNet));     //adds new network to the array
              }
          }
          netPos++;
      }

      return numSsid;

}

//function called when an interrupt occurs by the left contact sensor
void leftClash (void){
    lContact=true;                              //left contact
    posAvai=1;                                  //position is not available
    avoidCollision();                           //function that avoids the collision
}

//function called when an interrupt occurs by the right contact sensor
void rightClash (void){
    rContact=true;                              //right contact
    posAvai=1;                                  //position is not available
    avoidCollision();                           //function that avoids the collision

}

// functions that connects to the WIFI

void wifiConnect(void) {
    // attempt to connect to Wifi network:
    Serial.print("Attempting to connect to Network named: ");
    // print the network name (SSID);
    Serial.println(ssid);
    // Connect to WPA/WPA2 network. Change this line if using open or WEP network:
    WiFi.begin(ssid, password);
    while ( WiFi.status() != WL_CONNECTED) {
      // print dots while we wait to connect
      Serial.print(".");
      delay(300);
    }

    Serial.println("\nYou're connected to the network");
    Serial.println("Waiting for an ip address");

    while (WiFi.localIP() == INADDR_NONE) {
      // print dots while we wait for an ip addresss
      Serial.print(".");
      delay(300);
    }

    Serial.println("\nIP Address obtained");

    // you're connected now, so print out the status
    printWifiStatus();

    Serial.println("Starting webserver on port 80");
    server.begin();                           // start the web server on port 80
    Serial.println("Webserver started!");
}

// function that sends the network's SSID stored in the present session
void sendNetworks(void){

    int i=0;
    String data;

    while (ListOfNetworks[i].Id!=NULL) {
        data="idExp=" + String(idSession) + "&idNetw=" + String(ListOfNetworks[i].Id) + "&netwName="+ListOfNetworks[i].SSID+"\0"; //creation of the string to send data to the server
        Serial.println("data content:");
        Serial.println(data);


        Serial.println("Attempting to connect to server");  // information to the Serial: attempt to connect to the server

        //we are connected to the server by this point

        client.stop();      // close any connection before send a new request.
                            // This will free the socket on the WiFi shield

        if (client.connect(serverObj, port)) { // connection to the SERVER address and PORT specified
                Serial.println("\nConnected to the server!");
                  client.println("POST /AWSR/addNewNetworks.php HTTP/1.1");     //HTML header to send a POST message
                 client.print("Host: "); // SERVER ADDRESS
                 client.println(serverObj);
                 client.println("Content-Type: application/x-www-form-urlencoded"); //HTML header to specify the type of message
                 client.print("Content-Length: ");                                  //HTML header to specify the length of the message
                 client.println(data.length());
                 client.println();
                 client.println(data);


         } else { Serial.println("\nFailed when connecting to the server");}
         i++;
    }


    if (client.connected()) {
        client.stop();  // DISCONNECT FROM THE SERVER
    }

}

// function that sends the network's RSSI detected in the present session
void sendRSSI(void){

    int i=0;
    String data;

    while (actualRSSInetworks[i].Id!=NULL) {
        data="idExp=" + String(idSession) + "&idPos="+ String(idPosition) + "&idNetw="+ String(actualRSSInetworks[i].Id) + "&RSSI="+String(actualRSSInetworks[i].RSSI)+"\0"; //creation of the string to send data to the server
        Serial.println("RSSI content:");
        Serial.println(data);
        //trying to connect to the server
        // attempt to connect to the server
        Serial.println("Attempting to connect to server");

        //we are connected to the server by this point

        // close any connection before send a new request.
          // This will free the socket on the WiFi shield
        client.stop();

        if (client.connect(serverObj, port)) { // connection to the SERVER address and PORT specified
                Serial.println("\nConnected to the server!");
                  client.println("POST /AWSR/addPositionRSSI.php HTTP/1.1");        //HTML header to send a POST message
                 client.print("Host: "); // SERVER ADDRESS
                 client.println(serverObj);
                 client.println("Content-Type: application/x-www-form-urlencoded"); //HTML header to specify the type of message
                 client.print("Content-Length: ");                                  //HTML header to specify the length of the message
                 client.println(data.length());
                 client.println();
                 client.println(data);


         } else { Serial.println("\nFailed when connecting to the server");}
         i++;
    }


    if (client.connected()) {
        client.stop();  // DISCONNECT FROM THE SERVER
    }

}


//function that connects to retrySessionID.php to request the session Id from the database
int readSessionId(void) {
           client.stop();
           int idSes=0;
           char c[90];
           if (client.connect(serverObj, port)) { // connection to the SERVER address and PORT specified
                   Serial.println("\nConnected to the server in function readSessionId!");
                    client.println("GET /AWSR/retrySessionID.php HTTP/1.1");
                    client.print("Host: "); // SERVER ADDRESS
                    client.println(serverObj);
                    client.print("Connection:close\r\n\r\n");
                    while (client.connected()) {            // loop while the client's connected
                          if (client.available()) {             // if there's bytes to read from the client,
                              for (int k=0; k<90;k++) {
                                  c[k+1] = client.read();
                                  Serial.print(c[k+1]);
                               }
                              Serial.print("session id received: ");
                              for (int k=0; k<90;k++) {
                                  if (c[k]=='i' && c[k+1]=='d' && c[k+2]=='E' && c[k+3]=='x' && c[k+4]=='p') {     //looking for the chain idExp=''
                                      if (c[k+8]==39) {                                                            //when idExp contains one digit notice that char(39)='
                                              idSes=c[k+7]-'0';                                                         //idSes takes the value of the session
                                  } else if (c[k+9]==39){                                                           //if idExp contains two digits notice that char(39)='
                                              idSes=(c[k+7]-'0')*10 + (c[k+8]-'0');                                 //idSes takes the value of the session considering is two digits
                                  } else if (c[k+10]==39){                                                          //if idExp contains three digits notice that char(39)='
                                              idSes=(c[k+7]-'0')*100 + (c[k+8]-'0')*10+(c[k+9]-'0');
                              }
                              }}
                              Serial.println(idSes);
                          }}
            } else { Serial.println("\nFailed when connecting to the server");}

           client.stop();
           return idSes;
}

//function that determines the number of positions in the room
int detNumberPositions(void) {
    int posX, posY, posTOT;                 // number of positions in x and y
    posX=myDistSensor.distObstacle()/20;    //distance to obstacle divided by 20 cm (size of each position in X)
    mySCMD.moveMotors (RIGHT);                   //turns right
    delay(1000);                            //wait 1 second
    mySCMD.moveMotors (STOP);                   //stops
    posY=myDistSensor.distObstacle()/20;    //distance to obstacle divided by 20 cm (size of each position in Y)
    mySCMD.moveMotors (LEFT);                   //turns left
    delay(1000);                            //wait 1 second
    mySCMD.moveMotors (STOP);                   //stops
    posTOT=posX*posY;                       //total number of positions
    return posTOT;
}


// function that sends the network's SSID stored in the present session
void sendPosAvailable(void){

    String data;

    //while (ListOfNetworks[i].Id!=NULL) {
        data="idExp=" + String(idSession) + "&idPos="+ String(idPosition) + "&available="+ String(posAvai)+"\0"; //creation of the string to send data to the server
        Serial.println("data content:");
        Serial.println(data);


        Serial.println("Attempting to connect to server");  // information to the Serial: attempt to connect to the server

        //we are connected to the server by this point

        client.stop();      // close any connection before send a new request.
                            // This will free the socket on the WiFi shield

        if (client.connect(serverObj, port)) { // connection to the SERVER address and PORT specified
                Serial.println("\nConnected to the server!");
                  client.println("POST /AWSR/addPositionAva.php HTTP/1.1");         //HTML header to send a POST message
                 client.print("Host: "); // SERVER ADDRESS
                 client.println(serverObj);
                 client.println("Content-Type: application/x-www-form-urlencoded"); //HTML header to specify the type of message
                 client.print("Content-Length: ");                                  //HTML header to specify the length of the message
                 client.println(data.length());
                 client.println();
                 client.println(data);


         } else { Serial.println("\nFailed when connecting to the server");}

    if (client.connected()) {
        client.stop();  // DISCONNECT FROM THE SERVER
    }

}

//function that avoids a collision, called when the contact sensors are activated
void avoidCollision(void){
            mySCMD.moveMotors (STOP);                   //STOP both motors
            mySCMD.moveMotors (BACKWARD);                   //MOVE BACKWARD both motors
            delay(1000);                         // wait 2 seconds
            mySCMD.moveMotors (STOP);                   //STOP both motors
            mySCMD.moveMotors (RIGHT);                   //TURN RIGHT the robot
            delay(1000);                         // wait 2 seconds
            mySCMD.moveMotors (STOP);                   //STOP both motors

}
